
// cpresume v1.01 (C) 2001-2002 Mikael Klasson
// fluff@geocities.com
// http://mklasson.cjb.net
//
// Copies source file to dest file, resuming from dest file's end.
// If an offset is given, the data starting at OFFSET in source is appended to dest.
// Works fine with UNC paths (i.e. SMB networks), which is the reason I wrote it.
//
// Enjoy, this is Freeware. I take no responsibility whatsoever, yadda yadda...
//
// 2001-05-26, Fluff
// 2002-03-22, Fluff

#define	WIN32_LEAN_AND_MEAN

#include <stdio.h>
#include <stdlib.h>
#include <windows.h>

#define	CHUNK_SIZE	( 512 * 1024 )	// copy this many bytes at a time

unsigned char	abuff[CHUNK_SIZE];

void fix_unc_name( char *pdest, char *psrc )
{
	if( !strncmp( psrc, "\\\\", 2 ) ) {
		sprintf( pdest, "\\\\?\\UNC%s", psrc + 1 );
	} else {
		sprintf( pdest, psrc );
	}
}

int cpresume( HANDLE hs, HANDLE hd, int noffset )
{
	unsigned long	npos,nsize,nchunk,nres;

	if( ( nsize = GetFileSize( hs, NULL ) ) == 0xffffffff ) {
		printf( "couldn't get source file size\n" );
		return 1;
	}

	if( ( npos = GetFileSize( hd, NULL ) ) == 0xffffffff ) {
		printf( "couldn't get dest file size\n" );
		return 1;
	}

	if( SetFilePointer( hd, npos, NULL, FILE_BEGIN ) == 0xffffffff ) {
		printf( "couldn't seek in dest\n" );
		return 1;
	}
	if( noffset != -1 ) {
		npos = noffset;
	}
	if( SetFilePointer( hs, npos, NULL, FILE_BEGIN ) == 0xffffffff ) {
		printf( "couldn't seek in source\n" );
		return 1;
	}

	while( npos < nsize ) {
		nchunk = min( CHUNK_SIZE, nsize - npos );
		if( !ReadFile( hs, abuff, nchunk, &nres, NULL ) ) {
			printf( "couldn't read source\n" );
			return 1;
		}
		if( !WriteFile( hd, abuff, nchunk, &nres, NULL ) ) {
			printf( "couldn't write dest\n" );
			return 1;
		}
		npos += nchunk;
		printf( "\r%d / %d, %.0f%% ", npos, nsize, npos * 100.0f / nsize );
	}
	printf( "OK.\n" );
	return 0;
}

int main( int argc, char **argv )
{
	HANDLE		hs,hd;
	char		asrc[512];
	char		adest[512];
	int		nres;
	int		noffset = -1;

	if( argc < 3 ) {
		printf( "cpresume v1.01 by Mikael Klasson (fluff@geocities.com, http://mklasson.cjb.net)\n" );
		printf( "usage: cpresume <sourcefile> <destfile> [OFFSET]\n" );
		printf( "Copies sourcefile to destfile, resuming from destfile's end\n" );
		printf( "optionally starting at OFFSET in sourcefile\n" );
		return 0;
	}

	fix_unc_name( asrc, argv[1] );
	fix_unc_name( adest, argv[2] );

	if( ( hs = CreateFile( asrc, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, NULL ) ) == INVALID_HANDLE_VALUE ) {
		printf( "couldn't open source file\n" );
		return 1;
	}

	if( ( hd = CreateFile( adest, GENERIC_WRITE, FILE_SHARE_READ, NULL, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL ) ) == INVALID_HANDLE_VALUE ) {
		printf( "couldn't open destination file\n" );
		return 1;
	}

	if( argc > 3 ) {
		noffset = atoi( argv[3] );
	}

	nres = cpresume( hs, hd, noffset );

	CloseHandle( hd );
	CloseHandle( hs );

	return nres;
}
